﻿using System.Linq;
using HeuristicLab.Analysis.FitnessLandscape;
using HeuristicLab.Data;
using HeuristicLab.Problems.QuadraticAssignment;

namespace ProblemInstanceIdentifier {
  public class InstanceDescriptor {
    public string Name { get; set; }
    public string Cls { get; private set; }
    public int Dimension { get; set; }
    public double Sharpness { get; set; }
    public double SharpnessSdev { get; set; }
    public double Bumpiness { get; set; }
    public double BumpinessSdev { get; set; }
    public double Flatness { get; set; }
    public double FlatnessSdev { get; set; }
    public double Steadiness { get; set; }
    public double SteadinessSdev { get; set; }
    
    private InstanceDescriptor() { }

    public static InstanceDescriptor FromPathRelinkingAnalysis(QuadraticAssignmentProblem qap, int paths, int? seed) {
      var walk = new QAPDirectedWalk {
        Problem = qap,
        BestImprovement = true,
        Paths = paths,
        Seed = seed
      };
      var features = walk.Calculate().ToDictionary(x => x.Name, x => x.Value);
      
      return new InstanceDescriptor() {
        Name = qap.Name,
        Cls = GetClass(qap.Name),
        Dimension = qap.Weights.Rows,
        Sharpness = ((DoubleValue)features["Swap2.Sharpness"]).Value,
        Bumpiness = ((DoubleValue)features["Swap2.Bumpiness"]).Value,
        Flatness = ((DoubleValue)features["Swap2.Flatness"]).Value,
        Steadiness = ((DoubleValue)features["Swap2.Steadiness"]).Value,
      };
    }

    private static string GetClass(string name) {
      var cls = name.Substring(0, 3);
      var subCls = name.Last();
      switch (cls) {
        case "lip":
          cls = name.Substring(0, 4) + "-" + subCls;
          break;
        case "RAN":
          cls = name.Substring(0, 4) + "-" + name[name.Length - 2] + name[name.Length - 1];
          break;
        case "tai":
          if (char.IsLetter(subCls)) cls += "-" + subCls;
          break;
      }
      return cls;
    }

    public double CalculateSimilarity(InstanceDescriptor other) {
      var sa = (Sharpness - other.Sharpness);
      var bu = (Bumpiness - other.Bumpiness);
      var fl = (Flatness - other.Flatness);
      var se = (Steadiness - other.Steadiness);
      return sa * sa + bu * bu + fl * fl + se * se;
    }

    public override string ToString() {
      return Name;
    }
  }
}
